(document:surround "/std/frame")

;;; Functions

(define (read-all name)
  (woo-catch/message
    (lambda()
      (let ((data (woo-read-first "/net-iptables" 'name name)))
	(iface enumref "/net-iptables/avail_iface"
	       value (woo-get-option data 'name))
	(service enumref "/net-iptables/avail_service"
		 value (woo-get-option data 'service))
	(status value (woo-get-option data 'status))
	(tcp-ports value (woo-get-option data 'tcp_ports))
	(udp-ports value (woo-get-option data 'udp_ports))
	(forwarding value (woo-get-option data 'forwarding)
        (nat-status value (woo-get-option data 'nat_status))
	(nat-from enumref "/net-iptables/avail_network"
	          value (woo-get-option data 'nat_from))
	(nat-to enumref "/net-iptables/avail_iface"
	        value (woo-get-option data 'nat_to)))))))

(define (write-commit)
  (woo-catch/message
    (lambda()
      (woo-write "/net-iptables"
		 'commit #t
		 'name (iface value)
		 'service (service value)
		 'status (status value)
		 'tcp_ports (tcp-ports value)
		 'udp_ports (udp-ports value)
		 'forwarding (forwarding value)
		 'nat_status (nat-status value)
		 'nat_from (nat-from value)
		 'nat_to (nat-to value)))))

(define (write-reset)
  (woo-catch/message
    (lambda()
      (woo-write "/net-iptables"
		 'reset #t)
      (read-all ""))))

;;; UI

(gridbox
  columns "0;100"
  margin 20

  ;;
  (label text (bold (_ "General options")) colspan 2)

  ;;
  (label)
  (document:id status
	       (checkbox text (_ "Enable firewall")
			 name "status"))

  ;;
  (label)
  (document:id forwarding
	       (checkbox text (_ "Allow transit packets (forwarding)")
			 name "forwarding"))

  ;;
  (label)
  (hbox align "left"
	(document:id nat-status
		     (checkbox text (_ "Enable network address translation (NAT):")
			       name "nat_status"))
	(document:id nat-from (combobox name "nat_from"))
	(label text (_ "via:") name "nat_status")
	(document:id nat-to (combobox name "nat_from")))

  ;;
  (label colspan 2)

  ;;
  (label text (bold (_ "Allowed incoming connections")) colspan 2)

  ;;
  (label text (_ "Interface:") align "right" name "iface")
  (document:id iface (combobox
		       name "iface"
		       (when changed (read-all (iface value)))))


  (label)
  (document:id service (checklistbox name "service"))

  ;;
  (label colspan 2)

  ;;
  (label)
  (gridbox columns "20;80"
    (label align "right" text (_ "Open extra TCP ports:"))
    (document:id tcp-ports (edit name "tcp_ports"))
    (label)
    (label text (_ "(numbers separated by commas or spaces)"))

    (label align "right" text (_ "Open extra UDP ports:"))
    (document:id udp-ports (edit name "udp_ports"))
    (label)
    (label text (_ "(numbers separated by commas or spaces)"))
  )
  ;;
  (label colspan 2)

  ;;
  ;;
  (label)
  (hbox align "left"
	(button text (_ "Apply")
		(when clicked (write-commit)))
	(label "  ")
	(button text (_ "Restore default settings")
		(when clicked (write-reset)))))


(effect-disable "forwarding" "status" #f)

(effect-disable "nat_status" "status" #f)
(effect-disable "nat_from" "status" #f)
(effect-disable "nat_to" "status" #f)

(effect-disable "nat_status" "forwarding" #f)
(effect-disable "nat_from" "forwarding" #f)
(effect-disable "nat_to" "forwarding" #f)

(effect-disable "iface" "status" #f)
(effect-disable "service" "status" #f)

(effect-disable "tcp_ports" "status" #f)
(effect-disable "udp_ports" "status" #f)

(document:root
  (when loaded (read-all "")
               (init-effect)))
